#!/bin/bash
#

# build_epics.sh <product_directory> <compiler qualifier> [tar] [source]

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <compiler qualifier> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
maketar=${3}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=epics
origpkgver=v3_16_2
pkgver=${origpkgver}a
ssibuildshims_version=v1_04_13
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=base-${pkgdotver}

get_this_dir

get_ssibuildshims

source define_basics NO_TYPE

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# setup so we get the expected compiler
setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${PRODUCTS}

mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`
extra_command="-j ${ncore}"

if [[ "${basequal}" == e1[045] ]]; then
  compiler=gcc
  export CC=gcc
  export CXX=g++
  extra_command="stdver=c++14 ${extra_command}"
elif [[ "${basequal}" == e1[79] ]]; then
  compiler=gcc
  export CC=gcc
  export CXX=g++
  extra_command="stdver=c++17 ${extra_command}"
elif [[ "${basequal}" == e2[0] ]]; then
  compiler=gcc
  export CC=gcc
  export CXX=g++
  extra_command="stdver=c++17 ${extra_command}"
elif [[ "${basequal}" == c[27] ]]; then
  compiler=clang
  export CC=clang
  export CXX=clang++
  extra_command="stdver=c++17 ${extra_command}"
#elif [[ "${basequal}" == "i1" ]]; then
#  compiler=icc
#  extra_command="stdver=c++11 ${extra_command}"
else
  ssi_die "ERROR: unrecognized qualifier $basequal."
fi

set -x 

cd "${sourcedir}"
tar xf ${tardir}/${pkgtarname}.tar.gz || die "Unable to untar source."
cd ${pkgtarname}
export EPICS_HOST_ARCH=`./startup/EpicsHostArch`
export INSTALL_DIR=${pkgdir}
mkdir -p $INSTALL_DIR
cp -a ${patchdir}/* ${sourcedir}/${pkgtarname}
if [[ "${basequal}" == c[27] ]]; then
  mv ${sourcedir}/${pkgtarname}/configure/os/CONFIG_SITE.Common.linux-x86_64.clang ${sourcedir}/${pkgtarname}/configure/os/CONFIG_SITE.Common.linux-x86_64
else
  mv ${sourcedir}/${pkgtarname}/configure/os/CONFIG_SITE.Common.linux-x86_64.gcc ${sourcedir}/${pkgtarname}/configure/os/CONFIG_SITE.Common.linux-x86_64
fi

make compiler=${compiler} ${extra_command} || die "ERROR: make failed."

mv ${sourcedir}/${pkgtarname}/documentation ${sourcedir}/../doc
mv ${sourcedir}/${pkgtarname}/startup ${sourcedir}/../scripts

set +x

# Clean up.
cd "${pkgdir}"
chmod +x `find . -name *.pl`

# libtool and pkg-config files not useful. Also backups.
rm -rvf lib/pkgconfig lib/*.la
find . -name '*~' -exec rm -vf \{\} \;

${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "epics is installed at ${EPICS_FQ_DIR}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
